package com.sun.org.apache.bcel.internal.generic;

import com.sun.org.apache.bcel.internal.util.ByteSequence;
import java.io.IOException;
import java.io.DataOutputStream;

public abstract class Select extends BranchInstruction implements VariableLengthInstruction, StackConsumer, StackProducer
{
    private int[] match;
    private int[] indices;
    private InstructionHandle[] targets;
    private int fixed_length;
    private int match_length;
    private int padding;
    
    Select() {
        this.padding = 0;
    }
    
    Select(short opcode, int[] match, InstructionHandle[] targets, InstructionHandle defaultTarget) {
        super(opcode, null);
        this.padding = 0;
        this.match = match;
        this.targets = targets;
        super.setTarget(defaultTarget);
        for (InstructionHandle target2 : targets) {
            BranchInstruction.notifyTarget(null, target2, this);
        }
        if ((this.match_length = match.length) != targets.length) {
            throw new ClassGenException("Match and target array have not the same length: Match length: " + match.length + " Target length: " + targets.length);
        }
        this.indices = new int[this.match_length];
        return;
    }
    
    @Override
    protected int updatePosition(int offset, int max_offset) {
        this.setPosition(this.getPosition() + offset);
        short old_length = (short)super.getLength();
        this.padding = (4 - (this.getPosition() + 1) % 4) % 4;
        super.setLength((int)(short)(this.fixed_length + this.padding));
        return super.getLength() - old_length;
    }
    
    @Override
    public void dump(DataOutputStream out) throws IOException {
        out.writeByte((int)super.getOpcode());
        for (int i = 0; i < this.padding; ++i) {
            out.writeByte(0);
        }
        super.setIndex(this.getTargetOffset());
        out.writeInt(super.getIndex());
    }
    
    @Override
    protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
        this.padding = (4 - bytes.getIndex() % 4) % 4;
        for (int i = 0; i < this.padding; ++i) {
            bytes.readByte();
        }
        super.setIndex(bytes.readInt());
    }
    
    @Override
    public String toString(boolean verbose) {
        StringBuilder buf = new StringBuilder(super.toString(verbose));
        if (verbose) {
            for (int i = 0; i < this.match_length; ++i) {
                String s = "null";
                if (this.targets[i] != null) {
                    s = this.targets[i].getInstruction().toString();
                }
                buf.append("(").append(this.match[i]).append(", ").append(s).append(" = {").append(this.indices[i]).append("})");
            }
        }
        else {
            buf.append(" ...");
        }
        return buf.toString();
    }
    
    public void setTarget(int i, InstructionHandle target) {
        BranchInstruction.notifyTarget(this.targets[i], target, this);
        this.targets[i] = target;
    }
    
    @Override
    public void updateTarget(InstructionHandle old_ih, InstructionHandle new_ih) {
        boolean targeted = false;
        if (super.getTarget() == old_ih) {
            targeted = true;
            super.setTarget(new_ih);
        }
        for (int i = 0; i < this.targets.length; ++i) {
            if (this.targets[i] == old_ih) {
                targeted = true;
                this.setTarget(i, new_ih);
            }
        }
        if (!targeted) {
            throw new ClassGenException("Not targeting " + old_ih);
        }
        return;
    }
    
    @Override
    public boolean containsTarget(InstructionHandle ih) {
        if (super.getTarget() == ih) {
            return true;
        }
        InstructionHandle[] targets = this.targets;
        int length = targets.length;
        int i = 0;
        while (i < length) {
            InstructionHandle target2 = targets[i];
            if (target2 == ih) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    protected Object clone() throws CloneNotSupportedException {
        Select copy = (Select)super.clone();
        copy.match = (int[])this.match.clone();
        copy.indices = (int[])this.indices.clone();
        copy.targets = (InstructionHandle[])this.targets.clone();
        return copy;
    }
    
    @Override
    void dispose() {
        super.dispose();
        for (InstructionHandle target2 : this.targets) {
            target2.removeTargeter(this);
        }
    }
    
    public int[] getMatchs() {
        return this.match;
    }
    
    public int[] getIndices() {
        return this.indices;
    }
    
    public InstructionHandle[] getTargets() {
        return this.targets;
    }
    
    final int getMatch(int index) {
        return this.match[index];
    }
    
    final int getIndices(int index) {
        return this.indices[index];
    }
    
    final InstructionHandle getTarget(int index) {
        return this.targets[index];
    }
    
    final int getFixed_length() {
        return this.fixed_length;
    }
    
    final void setFixed_length(int fixed_length) {
        this.fixed_length = fixed_length;
    }
    
    final int getMatch_length() {
        return this.match_length;
    }
    
    final int setMatch_length(int match_length) {
        return this.match_length = match_length;
    }
    
    final void setMatch(int index, int value) {
        this.match[index] = value;
    }
    
    final void setIndices(int[] array) {
        this.indices = array;
    }
    
    final void setMatches(int[] array) {
        this.match = array;
    }
    
    final void setTargets(InstructionHandle[] array) {
        this.targets = array;
    }
    
    final int getPadding() {
        return this.padding;
    }
    
    final int setIndices(int i, int value) {
        return this.indices[i] = value;
    }
}
